/**********************************************************************************/
/*                                                                                */
/*                                                                                */
/* Program                                                                        */
/* =======                                                                        */
/*                                                                                */
/* mkaart.c                                                                       */
/*                                                                                */
/*                                                                                */
/* Purpose                                                                        */
/* =======                                                                        */
/*                                                                                */
/* To make POVRAY code for the "aart" object (see the aart*.pov files).           */
/* The code must be present inside a "sphere_sweep" POVRAY object.                */
/*                                                                                */
/*                                                                                */
/* Version 1.01                                                                   */
/* ============                                                                   */
/*                                                                                */
/* First version: September 2005                                                  */
/* Latest update: 2005-10-09                                                      */
/*                                                                                */
/*                                                                                */
/* Author of the program                                                          */
/* =====================                                                          */
/*                                                                                */
/* Stefan Spaennare, Lund, Sweden                                                 */
/*                                                                                */
/* E-mail: stefan@spaennare.se                                                    */
/*                                                                                */
/* Home page:   http://www.spaennare.se/index.html                                */
/*                                                                                */
/* POVRAY page: http://www.spaennare.se/povray.html                               */
/*                                                                                */
/*                                                                                */
/* Usage                                                                          */
/* =====                                                                          *********/
/*                                                                                        */
/* >Usage: mkaart r0r r1r s1r v0r w4r degpolyi n2pointspolyi npointsinterpi nlastpointsi  */
/*                                                                                        */
/* Parameters ending with "i" are integers and with "r" are real numbers          *********/
/* (i.e. double precision).                                                       */
/*                                                                                */
/* These parameters work well to define the "aart" object:                        */
/* -------------------------------------------------------                        */
/*                                                                                */
/* >mkaart 6.0 2.0 3.0 38.0 45.0 3 3 10 10 >temp                                  */
/*                                                                                */
/* The output code must be present inside a "sphere_sweep" POVRAY object          */
/* (see the aart*.pov files).                                                     */
/*                                                                                */
/* Explanation to the parameters:                                                 */
/* ------------------------------                                                 */
/*                                                                                */
/* r0: (6.0); Major radius of the toruses.                                        */
/*                                                                                */
/* r1: (2.0); Minor radius of the toruses.                                        */
/*                                                                                */
/* s1: (3.0); The distance between the centers of the two lower torus parts is    */
/*            (2.0*s1*sin(w4)).                                                   */
/*                                                                                */
/* v0: (38.0 deg); The angular circumference of the toruses is (360.0-2.0*v0)     */
/*                 degrees.                                                       */
/*                                                                                */
/* w4: (45.0 deg); Half the angle between two lower torus parts.                  */
/*                                                                                */
/* degpoly: (3); Degree of polynomials to be least squares fitted between ends of */
/*               torus parts.                                                     */
/*                                                                                */
/* n2pointspoly: (3); Half the number of points (at the end toruses) to be        */
/*                    least squares fitted to the polynomials. The following      */
/*                    must be valid: (2*n2pointspoly) >= (degpoly+1).             */
/*                                                                                */
/* npointsinterp: (10); Number of points to be interpolated with polynomials      */
/*                      in the gap between the torus end parts.                   */ 
/*                                                                                */
/* nlastpoints: (10); Number of last overlapping torus points to complete the     */
/*                    full object.                                                */
/*                                                                                */
/*                                                                                */
/* Built in functions                                                             */
/* ==================                                                             */
/*                                                                                */
/* Rotations:                                                                     */
/* ==========                                                                     */
/*                                                                                */
/* This C-program has built in functions for rotations in 3D around the           */
/* x-, y- or z-axis:                                                              */
/*                                                                                */
/* void rot_x(v1,v2,w)                                                            */
/* void rot_y(v1,v2,w)                                                            */
/* void rot_z(v1,v2,w)                                                            */
/*                                                                                */
/* Parameters (double precision):                                                 */
/*                                                                                */
/* v1: In-vector  (v1[1], v1[2], v1[3]) containing x, y and z.                    */
/* v2: Out-vector (v2[1], v2[2], v2[3]) containing x, y and z.                    */
/* w:  Rotation angle in radians.                                                 */
/*                                                                                */
/*                                                                                */
/* Least squares fit of polynomial:                                               */
/* ================================                                               */
/*                                                                                */
/* This C-program has a built in procedure to make a least squares fit of a       */
/* polynomial of arbitrary degree to two vectors containing x- and y-data.        */
/*                                                                                */
/* Example (polynomial of degree 3):                                              */
/*                                                                                */
/* y=cc[4]*x^3+cc[3]*x^2+cc[2]*x+cc[1];                                           */
/*                                                                                */
/* This function perform the polynomial least squares fit:                        */
/* -------------------------------------------------------                        */
/*                                                                                */
/* void makefit(vx,vy,na,deg,aa,aat,aaa,bbb,bb2,cc,t1,t2,t3)                      */
/*                                                                                */
/* This function returns a polynomial y-value if x (and the cc vector) is given:  */ 
/* -----------------------------------------------------------------------------  */
/*                                                                                */
/* double ccfunc(x,cc,deg)                                                        */
/*                                                                                */
/* Parameters (integer):                                                          */
/* ---------------------                                                          */
/*                                                                                */
/* deg: Polynomial degree (deg >= 0).                                             */
/* na: Number of data points in the x- and y-vectors (na >= (deg+1)).             */ 
/*                                                                                */
/* Parameters (double precision):                                                 */
/* ------------------------------                                                 */
/*                                                                                */
/* vx: Vector (array) containing data points in x (1, ..., na).                   */
/* vy: Vector (array) containing data points in y (1, ..., na).                   */
/* cc: Vector (array) containing polynomial coefficients (1, ..., (deg+1)).       */
/*                                                                                */
/* In the main code of this C-program the arrays vx and vy are declared and       */
/* allocated as vxx and vyy. The other arrays and matrices are allocated          */
/* automatically if deg and na are given (see the code in main). All the          */
/* parameters must be given to makefit when used.                                 */
/*                                                                                */
/* Important note. High order polynomials (deg > 3) tend to oscillate very much   */
/* if the points (vx and vy) do not fit exactly to the polynomial of degree       */
/* deg. This is however rarely the case when an over estimated least squares fit  */
/* is performed (i.e. when na > (deg+1)).                                         */
/*                                                                                */
/*                                                                                */
/* References                                                                     */
/* ==========                                                                     */
/*                                                                                */
/* The procedure "gaussj()" (Gauss-Jordan elimination) for inverting a matrix is  */
/* from Numerical Recipes in C Second Edition (Cambridge University Press 1992).  */
/*                                                                                */
/*                                                                                */
/**********************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#define swap(a,b) {temp=(a);(a)=(b);(b)=temp;}

double timetic=(double)(CLOCKS_PER_SEC);

double pi=3.14159265358979323;


void gaussj(a,n,b,m,indxc,indxr,ipiv)
double **a;
int n;
double **b;
int m;
int *indxc;
int *indxr;
int *ipiv;
{
   int i,icol,irow,j,k,l,ll;
   double big,dum,pivinv,temp;

   for (j=1; j<=n; j++) {
       ipiv[j]=0;
   } /* for j */

   for (i=1; i<=n; i++) {
      big=0.0;
      for (j=1; j<=n; j++) {
         if (ipiv[j] != 1) {
            for (k=1; k<=n; k++) {
               if (ipiv[k] == 0) {
                  if (fabs(a[j][k]) >= big) {
                     big=fabs(a[j][k]);
                     irow=j;
                     icol=k;
                  } /* if */
               }
               else if (ipiv[k] > 1) {
                    printf("Singular matrix 1\n");
                    exit(0);
               } /* else */
            } /* for k */
         } /* if */
      } /* for j */
      ipiv[icol]=ipiv[icol]+1;
      if (irow != icol) {
         for (l=1; l<=n; l++) {
             swap(a[irow][l],a[icol][l]);
         } /* for l */
         for (l=1; l<=m; l++) {
             swap(b[irow][l],b[icol][l]);
         } /* for l */
      } /* if */
      indxr[i]=irow;
      indxc[i]=icol;
      if (a[icol][icol] == 0.0) {
         printf("Singular matrix 2\n");
         exit(0);
      } /* if */
      pivinv=1.0/a[icol][icol];
      a[icol][icol]=1.0;
      for (l=1; l<=n; l++) {
          a[icol][l]=pivinv*a[icol][l];
      } /* for l */
      for (l=1; l<=m; l++) {
          b[icol][l]=pivinv*b[icol][l];
      } /* for l */
      for (ll=1; ll<=n; ll++) {
          if (ll != icol) {
             dum=a[ll][icol];
             a[ll][icol]=0.0;
             for (l=1; l<=n; l++) {
                 a[ll][l]=a[ll][l]-a[icol][l]*dum;
             } /* for l */
             for (l=1; l<=m; l++) {
                 b[ll][l]=b[ll][l]-b[icol][l]*dum;
             } /* for l */
          } /* if */
      } /* for ll */
   } /* for i */
  
   for (l=n; l>=1; l--) {
       if (indxr[l] != indxc[l]) {
          for (k=1; k<=n; k++) {
              swap(a[k][indxr[l]],a[k][indxc[l]]);
          } /* for k */
       } /* if */
   } /* for l */

} /* gaussj */
   

void makefit(vx,vy,na,deg,aa,aat,aaa,bbb,bb2,cc,t1,t2,t3)
double *vx;
double *vy;
int na;
int deg;
double **aa;
double **aat;
double **aaa;
double **bbb;
double *bb2;
double *cc;
int *t1,*t2,*t3;
{
   int i,j,k,n0,m0;
   double x,temp,sum;

   m0=1;

   n0=deg+1;


   for (k=1; k<=na; k++) {

      x=vx[k];

      temp=1.0;

      for (i=1; i<=n0; i++) {

         aa[k][i]=temp; 
	 aat[i][k]=temp;

	 temp=temp*x;

      } /* for i */


      bb2[k]=vy[k];

   } /* for k */


   for (i=1; i<=n0; i++) {
      for (j=1; j<=n0; j++) {
         sum=0.0;
         for (k=1; k<=na; k++) {
            sum=sum+aat[i][k]*aa[k][j];
         } /* for k */
         aaa[i][j]=sum;
      } /* for j */
   } /* for i */ 


   for (j=1; j<=n0; j++) {
      sum=0.0;
      for (k=1; k<=na; k++) {
         sum=sum+aat[j][k]*bb2[k];
      } /* for k */
      bbb[j][1]=sum;
   } /* for j */


   gaussj(aaa,n0,bbb,m0,t1,t2,t3);


   for (i=1; i<=n0; i++) {
      cc[i]=bbb[i][1];
   } /* for i */


} /* makefit */ 


double ccfunc(x,cc,deg)
double x;
double *cc;
int deg;
{

   int i,n0;
   double temp,t;

   n0=deg+1;

   temp=0.0;
   t=1.0;

   for (i=1; i<=n0; i++) {
      temp=temp+cc[i]*t;
      t=t*x;
   } /* for i */

   return(temp);

} /* ccfunc */


double deg(x)
double x;
{
   return(x*180.0/pi);
} /* deg */


double rad(x)
double x;
{
   return(x*pi/180.0);
} /* rad */


void rot_x(v1,v2,w)
double *v1;
double *v2;
double w;
{
  double cw,sw;

  cw=cos(w);
  sw=sin(w);

  v2[1]=v1[1];
  v2[2]=v1[2]*cw-v1[3]*sw;
  v2[3]=v1[2]*sw+v1[3]*cw;

} /* rot_x */


void rot_y(v1,v2,w)
double *v1;
double *v2;
double w;
{
  double cw,sw;

  cw=cos(w);
  sw=sin(w);

  v2[1]=v1[1]*cw+v1[3]*sw;
  v2[2]=v1[2];
  v2[3]=-v1[1]*sw+v1[3]*cw;

} /* rot_y */


void rot_z(v1,v2,w)
double *v1;
double *v2;
double w;
{
  double cw,sw;

  cw=cos(w);
  sw=sin(w);

  v2[1]=v1[1]*cw-v1[2]*sw;
  v2[2]=v1[1]*sw+v1[2]*cw;
  v2[3]=v1[3];

} /* rot_z */


int main(argc,argv)
int argc;
char *argv[];

{

   int i,k,vmin,vmax,vstep,nr,nn;
   int deg1,deg2,naa,maa,na,mm,np,ni,nl;

   double r0,r1,v,x,y,z,z1; 
   double cv,sv,v0,c1,s1;
   double w1,w2,w3,w4,qqx,qqy,qqz,x2,y2,z2,t;

   double aa1[4];
   double bb1[4];
   double cc1[4];
   double dd1[4];
   double ee1[4];

   double *xx0,*yy0,*zz0;
   double *xx1,*yy1,*zz1;
   double *xx2,*yy2,*zz2;
   double *xx3,*yy3,*zz3;

   double *vxx;
   double *vyy;

   clock_t time1,timediff1;

   double **aa,**aat;
   double *mpaa, *mpaat;

   double **aaa;
   double  *mpaaa;

   double **bbb;
   double  *mpbbb;

   double *cc_x;
   double *cc_y;
   double *cc_z;

   double *bb2;

   int *t1,*t2,*t3;


   if (argc != 10) {
      printf("Usage: %s r0r r1r s1r v0r w4r degpolyi n2pointspolyi npointsinterpi nlastpointsi\n",argv[0]);
      exit(0);
   } /* if */


   r0=atof(argv[1]);
   r1=atof(argv[2]);
   s1=atof(argv[3]);
   v0=atof(argv[4]);
   w4=atof(argv[5]);
   deg1=atoi(argv[6]);
   np=atoi(argv[7]);
   ni=atoi(argv[8]);
   nl=atoi(argv[9]);

   if (deg1 < 0) {
      deg1=0;
   } /* if */ 

   if (np < (deg1/2+1)) {
      np=deg1/2+1;
   } /* if */

   deg2=deg1;

   if (deg1 > 3) {
      deg2=3;
   } /* if */


/* Set up torus arrays. ***********************************************/

   nn=2*(180/10)+2*np+3;

   xx0=(double *)calloc(nn+1,sizeof(double));
   yy0=(double *)calloc(nn+1,sizeof(double));
   zz0=(double *)calloc(nn+1,sizeof(double));

   xx1=(double *)calloc(nn+1,sizeof(double));
   yy1=(double *)calloc(nn+1,sizeof(double));
   zz1=(double *)calloc(nn+1,sizeof(double));

   xx2=(double *)calloc(nn+1,sizeof(double));
   yy2=(double *)calloc(nn+1,sizeof(double));
   zz2=(double *)calloc(nn+1,sizeof(double));

   xx3=(double *)calloc(nn+1,sizeof(double));
   yy3=(double *)calloc(nn+1,sizeof(double));
   zz3=(double *)calloc(nn+1,sizeof(double));


/* Set up polynomial least square fit arrays. **************************/

naa=deg1+1;

maa=2*np;

vxx=(double *)calloc(maa+1,sizeof(double));
vyy=(double *)calloc(maa+1,sizeof(double));

t1=(int *)calloc(naa+1,sizeof(int));
t2=(int *)calloc(naa+1,sizeof(int));
t3=(int *)calloc(naa+1,sizeof(int));

aaa=(double **)calloc(naa+1,sizeof(double*));
mpaaa=(double *)calloc((naa+1)*(naa+1),sizeof(double));

for (i=0; i<=naa; i++) {
   aaa[i]=mpaaa;
   mpaaa=mpaaa+naa+1;
} /* for i */

bbb=(double **)calloc(naa+1,sizeof(double*));
mpbbb=(double *)calloc((naa+1)*2,sizeof(double));

for (i=0; i<=naa; i++) {
   bbb[i]=mpbbb;
   mpbbb=mpbbb+2;
} /* for i */

aa=(double **)calloc(maa+1,sizeof(double*));
aat=(double **)calloc(naa+1,sizeof(double*));

mpaa=(double *)calloc((maa+1)*(naa+1),sizeof(double));
mpaat=(double *)calloc((maa+1)*(naa+1),sizeof(double));

for (i=0; i<=maa; i++) {
   aa[i]=mpaa;
   mpaa=mpaa+naa+1;
} /* for i */

for (i=0; i<=naa; i++) {
   aat[i]=mpaat; 
   mpaat=mpaat+maa+1;
} /* for i */

cc_x=(double *)calloc(naa+1,sizeof(double));
cc_y=(double *)calloc(naa+1,sizeof(double));
cc_z=(double *)calloc(naa+1,sizeof(double));

bb2=(double *)calloc(maa+1,sizeof(double));


/* Set initial torus parameters. ***********************************/


   cv=cos(rad(v0));
   sv=sin(rad(v0));

   c1=(180.0-v0)/180.0;

   z1=s1*(1.0-v0/180.0);

   k=0;

/* Compute twisted torus part 1. --------------------------------------*/

   vmin=-180;
   vmax=-180+5*(np-1);
   vstep=5;

   for (i=vmin; i<=vmax; i=i+vstep) {

      k++;

      v=(double)(i)*c1*pi/180.0;

      z=-z1*(double)(i)/180.0;

      x=r0*cos(v);
      y=r0*sin(v);

      xx0[k]=x;
      yy0[k]=y;
      zz0[k]=z;

   } /* for i */

/* Compute twisted torus part 2. --------------------------------------*/

   vmin=-180+5*(np+1);
   vmax=180-5*(np+1);
   vstep=10;

   for (i=vmin; i<=vmax; i=i+vstep) {

      k++;

      v=(double)(i)*c1*pi/180.0;

      z=-z1*(double)(i)/180.0;

      x=r0*cos(v);
      y=r0*sin(v);

      xx0[k]=x;
      yy0[k]=y;
      zz0[k]=z;

   } /* for i */

/* Compute twisted torus part 3. --------------------------------------*/

   vmin=180-5*(np-1);
   vmax=180;
   vstep=5;

   for (i=vmin; i<=vmax; i=i+vstep) {

      k++;

      v=(double)(i)*c1*pi/180.0;

      z=-z1*(double)(i)/180.0;

      x=r0*cos(v);
      y=r0*sin(v);

      xx0[k]=x;
      yy0[k]=y;
      zz0[k]=z;

   } /* for i */

/*----------------------------------------------------------------------------*/

   mm=k; 


/* Rotate three twisted torus parts to correct positions. *********************/

   w1=90.0;
   w2=90.0;
   w3=(180.0-v0);

   k=mm;

   for (i=1; i<=mm; i++) {

      aa1[1]=xx0[k];
      aa1[2]=yy0[k];
      aa1[3]=zz0[k];

      rot_z(aa1,bb1,rad(w1));

      xx1[i]=bb1[1];
      yy1[i]=bb1[2];
      zz1[i]=bb1[3];

      k--;

   } /* for i */
  

   k=1;

   for (i=1; i<=mm; i++) {

      aa1[1]=xx0[k];
      aa1[2]=yy0[k];
      aa1[3]=zz0[k];

      rot_z(aa1,bb1,rad(w1));
      rot_y(bb1,cc1,rad(w2));
      rot_x(cc1,dd1,-rad(w3));
      rot_z(dd1,ee1,-rad(w4));

      xx2[i]=ee1[1]-s1*sin(rad(w4));
      yy2[i]=ee1[2]-r0;
      zz2[i]=ee1[3]-s1;

      k++;

   } /* for i */
  

   k=mm;

   for (i=1; i<=mm; i++) {

      aa1[1]=xx0[i];
      aa1[2]=yy0[i];
      aa1[3]=zz0[i];

      rot_z(aa1,bb1,rad(w1));
      rot_y(bb1,cc1,-rad(w2));
      rot_x(cc1,dd1,rad(w3));
      rot_z(dd1,ee1,rad(w4));

      xx3[k]=ee1[1]+s1*sin(rad(w4));
      yy3[k]=ee1[2]-r0;
      zz3[k]=ee1[3]+s1;

      k--;

   } /* for i */


/* Print coordinates and least square fitted connections between twisted toruses. */

/* Print twisted torus 1. --------------------------------------------------------*/

   nr=3*mm+3*ni+nl;

   printf("%8d,\n",nr);

   printf("\n");

   for (i=1; i<=mm; i++) {

      printf("<%16.12f, %16.12f, %16.12f>, %16.12f\n",xx1[i],yy1[i],zz1[i],r1);

   } /* for i */

   printf("\n");

/* Polynomial least squares fit 1 and print connection between twisted toruses. */

   k=1;

   for (i=(mm-np+1); i<=mm; i++) {
      vxx[k]=xx1[i];
      vyy[k]=yy1[i];
      k++;
   } /* for i */
      
   for (i=1; i<=np; i++) {
      vxx[k]=xx2[i];
      vyy[k]=yy2[i];
      k++;
   } /* for i */

   na=k-1;

   makefit(vxx,vyy,na,deg1,aa,aat,aaa,bbb,bb2,cc_x,t1,t2,t3);

   k=1;

   for (i=(mm-np+1); i<=mm; i++) {
      vxx[k]=yy1[i];
      vyy[k]=zz1[i];
      k++;
   } /* for i */
      
   for (i=1; i<=np; i++) {
      vxx[k]=yy2[i];
      vyy[k]=zz2[i];
      k++;
   } /* for i */

   na=k-1;

   makefit(vxx,vyy,na,deg1,aa,aat,aaa,bbb,bb2,cc_y,t1,t2,t3);

   k=1;

   for (i=(mm-np+1); i<=mm; i++) {
      vxx[k]=zz1[i];
      vyy[k]=xx1[i];
      k++;
   } /* for i */
      
   for (i=1; i<=np; i++) {
      vxx[k]=zz2[i];
      vyy[k]=xx2[i];
      k++;
   } /* for i */

   na=k-1;

   makefit(vxx,vyy,na,deg1,aa,aat,aaa,bbb,bb2,cc_z,t1,t2,t3);

   qqx=(xx2[1]-xx1[mm])/(double)(ni+1);
   qqy=(yy2[1]-yy1[mm])/(double)(ni+1);
   qqz=(zz2[1]-zz1[mm])/(double)(ni+1);

   for (i=1; i<=ni; i++) {

      t=(double)(i);

      x=xx1[mm]+qqx*t;
      y=yy1[mm]+qqy*t;
      z=zz1[mm]+qqz*t;

      y2=ccfunc(x,cc_x,deg1);
      z2=ccfunc(y,cc_y,deg1);
      x2=ccfunc(z,cc_z,deg1);
      
      printf("<%16.12f, %16.12f, %16.12f>, %16.12f\n",x2,y2,z2,r1);

   } /* for i */
   

/* Print twisted torus 2. --------------------------------------------------------*/

   printf("\n");

   for (i=1; i<=mm; i++) {

      printf("<%16.12f, %16.12f, %16.12f>, %16.12f\n",xx2[i],yy2[i],zz2[i],r1);

   } /* for i */

   printf("\n");


/* Polynomial least squares fit 2 and print connection between twisted toruses. */

   k=1;

   for (i=(mm-np+1); i<=mm; i++) {
      vxx[k]=xx2[i];
      vyy[k]=yy2[i];
      k++;
   } /* for i */
      
   for (i=1; i<=np; i++) {
      vxx[k]=xx3[i];
      vyy[k]=yy3[i];
      k++;
   } /* for i */

   na=k-1;

   makefit(vxx,vyy,na,deg2,aa,aat,aaa,bbb,bb2,cc_x,t1,t2,t3);

   k=1;

   for (i=(mm-np+1); i<=mm; i++) {
      /*
      vxx[k]=yy2[i];
      */
      vxx[k]=xx2[i];
      vyy[k]=zz2[i];
      k++;
   } /* for i */
      
   for (i=1; i<=np; i++) {
      /*
      vxx[k]=yy3[i];
      */
      vxx[k]=xx3[i];
      vyy[k]=zz3[i];
      k++;
   } /* for i */

   na=k-1;

   makefit(vxx,vyy,na,deg2,aa,aat,aaa,bbb,bb2,cc_y,t1,t2,t3);

   k=1;

   for (i=(mm-np+1); i<=mm; i++) {
      vxx[k]=zz2[i];
      vyy[k]=xx2[i];
      k++;
   } /* for i */
      
   for (i=1; i<=np; i++) {
      vxx[k]=zz3[i];
      vyy[k]=xx3[i];
      k++;
   } /* for i */

   na=k-1;

   makefit(vxx,vyy,na,deg1,aa,aat,aaa,bbb,bb2,cc_z,t1,t2,t3);

   qqx=(xx3[1]-xx2[mm])/(double)(ni+1);
   qqy=(xx3[1]-xx2[mm])/(double)(ni+1);
   /*
   qqy=(yy3[1]-yy2[mm])/(double)(ni+1);
   */
   qqz=(zz3[1]-zz2[mm])/(double)(ni+1);

   for (i=1; i<=ni; i++) {

      t=(double)(i);

      x=xx2[mm]+qqx*t;
      y=xx2[mm]+qqx*t;
      /*
      y=yy2[mm]+qqy*t;
      */
      z=zz2[mm]+qqz*t;

      y2=ccfunc(x,cc_x,deg2);
      z2=ccfunc(y,cc_y,deg2);
      x2=ccfunc(z,cc_z,deg1);
      
      printf("<%16.12f, %16.12f, %16.12f>, %16.12f\n",x2,y2,z2,r1);

   } /* for i */


/* Print twisted torus 3. --------------------------------------------------------*/


   printf("\n");

   for (i=1; i<=mm; i++) {

      printf("<%16.12f, %16.12f, %16.12f>, %16.12f\n",xx3[i],yy3[i],zz3[i],r1);

   } /* for i */

   printf("\n");


/* Polynomial least squares fit 3 and print connection between twisted toruses. */

   k=1;

   for (i=(mm-np+1); i<=mm; i++) {
      vxx[k]=xx3[i];
      vyy[k]=yy3[i];
      k++;
   } /* for i */
      
   for (i=1; i<=np; i++) {
      vxx[k]=xx1[i];
      vyy[k]=yy1[i];
      k++;
   } /* for i */

   na=k-1;

   makefit(vxx,vyy,na,deg1,aa,aat,aaa,bbb,bb2,cc_x,t1,t2,t3);

   k=1;

   for (i=(mm-np+1); i<=mm; i++) {
      vxx[k]=yy3[i];
      vyy[k]=zz3[i];
      k++;
   } /* for i */
      
   for (i=1; i<=np; i++) {
      vxx[k]=yy1[i];
      vyy[k]=zz1[i];
      k++;
   } /* for i */

   na=k-1;

   makefit(vxx,vyy,na,deg1,aa,aat,aaa,bbb,bb2,cc_y,t1,t2,t3);

   k=1;

   for (i=(mm-np+1); i<=mm; i++) {
      vxx[k]=zz3[i];
      vyy[k]=xx3[i];
      k++;
   } /* for i */
      
   for (i=1; i<=np; i++) {
      vxx[k]=zz1[i];
      vyy[k]=xx1[i];
      k++;
   } /* for i */

   na=k-1;

   makefit(vxx,vyy,na,deg1,aa,aat,aaa,bbb,bb2,cc_z,t1,t2,t3);

   qqx=(xx1[1]-xx3[mm])/(double)(ni+1);
   qqy=(yy1[1]-yy3[mm])/(double)(ni+1);
   qqz=(zz1[1]-zz3[mm])/(double)(ni+1);

   for (i=1; i<=ni; i++) {

      t=(double)(i);

      x=xx3[mm]+qqx*t;
      y=yy3[mm]+qqy*t;
      z=zz3[mm]+qqz*t;

      y2=ccfunc(x,cc_x,deg1);
      z2=ccfunc(y,cc_y,deg1);
      x2=ccfunc(z,cc_z,deg1);
      
      printf("<%16.12f, %16.12f, %16.12f>, %16.12f\n",x2,y2,z2,r1);

   } /* for i */


/* Print first part of twisted torus 1 again. -----------------------------------*/


   printf("\n");

   for (i=1; i<=nl; i++) {

      printf("<%16.12f, %16.12f, %16.12f>, %16.12f\n",xx1[i],yy1[i],zz1[i],r1);

   } /* for i */


/***************************************************************************/


} /* End */
