/**********************************************************************************/
/*                                                                                */
/*                                                                                */
/* Program                                                                        */
/* =======                                                                        */
/*                                                                                */
/* mkcoshyp.c                                                                     */
/*                                                                                */
/*                                                                                */
/* Purpose                                                                        */
/* =======                                                                        */
/*                                                                                */
/* To make POVRAY code for the cosine hyperbolic chain (see the coshyp.pov file). */
/*                                                                                */
/*                                                                                */
/* Version 1.03                                                                   */
/* ============                                                                   */
/*                                                                                */
/* First version: 2006-08-19                                                      */
/* Latest update: 2006-08-22                                                      */
/*                                                                                */
/*                                                                                */
/* Author of the program                                                          */
/* =====================                                                          */
/*                                                                                */
/* Stefan Spaennare, Lund, Sweden                                                 */
/*                                                                                */
/* E-mail: stefan@spaennare.se                                                    */
/*                                                                                */
/* Home page:   http://www.spaennare.se/index.html                                */
/*                                                                                */
/* POVRAY page: http://www.spaennare.se/povray.html                               */
/*                                                                                */
/*                                                                                */
/* Usage                                                                          */
/* =====                                                                          */
/*                                                                                */
/* Usage: mkcoshyp outfile n_inti x1r x2r width_facr n_linksi scaler y_trr z_trr  */
/*                                                                                */
/* Parameters ending with "i" are integers and with "r" are real numbers          */
/* (i.e. double precision).                                                       */
/*                                                                                */
/* These parameters work well to make a POVRAY chain with 33 links:               */
/* ----------------------------------------------------------------               */
/*                                                                                */
/* mkcoshyp ch.pov 1000 -4.0 4.0 2.0 33 1.7 -8.25 4.0                             */
/*                                                                                */
/* Explanation of the parameters:                                                 */
/* ------------------------------                                                 */
/*                                                                                */
/* outfile: (ch.pov); Outfile with POVRAY code.                                   */
/*                                                                                */
/* n_int: (1000); Number of integration steps for the cos hyp curve.              */
/*                                                                                */
/* x1: (-4.0); Start x-value for the cos hyp curve.                               */
/*                                                                                */
/* x2: (4.0);  Stop x-value for the cos hyp curve.                                */
/*                                                                                */
/* width_fac: (2.0); Sets the width factor (1/k) of the cos hyp curve.            */
/*                                                                                */
/* n_links: (33); Number of links in the chain (odd number).                      */
/*                                                                                */
/* scale: (1.7);  Scale factor for the chain in the generated POVRAY code.        */
/*                                                                                */
/* y_tr: (-8.25); y-translation for the chain in the generated POVRAY code.       */
/*                                                                                */
/* z_tr: (4.0);   z-translation for the chain in the generated POVRAY code.       */
/*                                                                                */
/*                                                                                */
/* The cosine hyperbolic function                                                 */
/* ==============================                                                 */
/*                                                                                */
/* The cosine hyperbolic function on the form y = (1/k)*cosh(k*x) defines         */
/* among other things the shape of a hanging chain or wire (under the force of    */
/* gravity) with both ends fixed.                                                 */
/*                                                                                */
/*                                                                                */
/**********************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/


#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

double timetic=(double)(CLOCKS_PER_SEC);

double pi=3.14159265358979323;


double sgn(x)
double x;
{
   if (x >= 0.0) {
      return(1.0);
   }
   else {
      return(-1.0);
   } /* if */
} /* sgn */


double deg(x)
double x;
{
   return(x*180.0/pi);
} /* deg */


/* Integrates the length of the cos hyp curve from x01 to x02. */

double int_coshyp(x01,x02,c,nint)
double x01;
double x02;
double c;
int nint;
{
   int i;
   double cinv,x1,y1,x2,y2,r,s,sum,xx,yy;

   cinv=1.0/c;

   s=(x02-x01)/(double)(nint);

   x1=x01;
   y1=cinv*cosh(c*x01);

   sum=0.0;

   for (i=1; i<=nint; i++) {

      x2=x01+s*(double)(i);
      y2=cinv*cosh(c*x2);

      xx=x2-x1;
      yy=y2-y1;

      r=sqrt(xx*xx+yy*yy);

      sum=sum+r;

      x1=x2;
      y1=y2;

   } /* for i */

   return(sum);

} /* int_coshyp */


/* Returns the differnce between the length between two points on the      */
/* cos hyp curve (with x-coordinates x01 and x02) and the link length (l). */

double f_link(x01,x02,c,l)
double x01;
double x02;
double c;
double l;
{
   double cinv,x1,y1,x2,y2,r,xx,yy;

   cinv=1.0/c;

   x1=x01;
   y1=cinv*cosh(c*x01);

   x2=x02;
   y2=cinv*cosh(c*x02);
      
   xx=x2-x1;
   yy=y2-y1;

   r=sqrt(xx*xx+yy*yy);

   return(r-l);

} /* f_link */


/* Asumed a link length of (l) this function calculates the link      */
/* intersection points on the cos hyp curve from x01 to x02 using     */
/* interval bisection. The final difference from x = x02 is returned. */

double ib_links(xx1,yy1,x01,x02,c,l,eps,nint,nlinks)
double *xx1;
double *yy1;
double x01;
double x02;
double c;
double l;
double eps;
int nint;
int nlinks;
{

   int i,j;

   double cinv,r,s,x1,y1,x2,y2,xx,yy;
   double a,b,x,x0,l0;

   cinv=1.0/c;

   s=(x02-x01)/(double)(nint);

   x=x01;

   x1=x;
   y1=cinv*cosh(c*x);

   j=1;

   xx1[j]=x1;
   yy1[j]=y1;


   for (i=1; i<=nlinks; i++) {   

      if ((i % 2) == 1) {
         l0=l;
      }
      else {
         l0=2.0*l;
      } /* if */

      x0=x;

      x2=x0+s;

      do {

         y2=cinv*cosh(c*x2);

         xx=x2-x1;
         yy=y2-y1;

         r=sqrt(xx*xx+yy*yy);

         x2=x2+s;

       } while (r <= l0);


       a=x2-2.0*s;
       b=x2;
       x=1.0;


       while ((fabs(a-b)) > (fabs(eps*x))) { 

          x=0.5*(a+b);

          if ((f_link(x0,a,c,l0)*
               f_link(x0,x,c,l0)) > 0.0) {
             a=x;
          }
          else {
             b=x;
          } /* if */

       } /* while */


       x1=x;
       y1=cinv*cosh(c*x);

      j++;

      xx1[j]=x1;
      yy1[j]=y1;

   } /* for i */


   x2=x02;
   y2=cinv*cosh(c*x02);

   xx=x2-x1;
   yy=y2-y1;

   r=sqrt(xx*xx+yy*yy);

/*
   printf("r = %16.12f\n",r);
*/

   return(sgn(xx)*r);

} /* ib_links */


int main(argc,argv)
int argc;
char *argv[];
{

   FILE *outfile;

   int i,nint,nlinks,nl;

   double r,c,c0,l,x01,x02,sum,eps;
   double a,b,x,x1,y1,x2,y2,xx,yy,w;
   double ytr,ztr,scale1;

   double *xx1;
   double *yy1;


   if (argc != 10) {
      printf("Usage: %s outfile n_inti x1r x2r width_facr n_linksi scaler y_trr z_trr\n",argv[0]);
      exit(0);
   } /* if */

   nint=atoi(argv[2]);
   x01=atof(argv[3]);
   x02=atof(argv[4]);
   c0=atof(argv[5]);
   nlinks=atoi(argv[6]);
   scale1=atof(argv[7]);
   ytr=atof(argv[8]);
   ztr=atof(argv[9]);

   outfile=fopen(argv[1],"w");

   xx1=(double *)calloc(nlinks+3,sizeof(double));
   yy1=(double *)calloc(nlinks+3,sizeof(double));

   c=1.0/c0;

   eps=1.0e-14;

   if (nlinks < 3) {
      nlinks=3;
   } /* if */

   nlinks=(nlinks-1)/2;
   nlinks=2*nlinks+1;

   nl=(nlinks-1)/2;

   if ((10*nlinks) > nint) {
      nint=10*nlinks;
   } /* if */

   printf("\n");
   printf("POVRAY cosine hyperbolic chain\n");
   printf("==============================\n");
   printf("\n");

   printf("n_links = %6d\n",nlinks);
   printf("\n");
   printf("n_int =   %6d\n",nint);

   sum=int_coshyp(x01,x02,c,nint);

   printf("\n");
   printf("int_sum =    %18.14f\n",sum);
   printf("\n");

   if ((nl % 2) == 0) {
      l=sum/(3.0*(double)(nl)+1.0);
   }
   else {
      l=sum/(3.0*(double)(nl-1)+4.0);
   } /* if */

   printf("l_0 =        %18.14f\n",l);

   a=0.75*l;
   b=1.25*l;
   x=1.0;


/* This loop improves the final value of the link length (l) */
/* to desired accuracy using interval bisection.             */

    while ((fabs(a-b)) > (fabs(eps*x))) { 

       x=0.5*(a+b);
/*
       printf("x = %16.14f\n",x);
*/

       if ((ib_links(xx1,yy1,x01,x02,c,a,eps,nint,nlinks)*
            ib_links(xx1,yy1,x01,x02,c,x,eps,nint,nlinks)) > 0.0) {
          a=x;
       }
       else {
          b=x;
       } /* if */

    } /* while */


   printf("\n");
   printf("l_final =    %18.14f\n",x);
   printf("\n");


   x1=xx1[1];
   y1=yy1[1];

   printf("   i     x(i)               y(i)               r(i,i-1)\n");

   for (i=1; i<=(nlinks+1); i++) {

      x2=xx1[i];
      y2=yy1[i];

      xx=x2-x1;
      yy=y2-y1;

      r=sqrt(xx*xx+yy*yy);

      printf("%4d %18.14f %18.14f %18.14f\n",i,xx1[i],yy1[i],r);

      x1=x2;
      y1=y2;

   } /* for i */

   printf("\n");
   printf("Output to file %s.\n",argv[1]);
   printf("\n");


/* Print final POVRAY code for the chain to the outfile. */

   fprintf(outfile,"#declare s1 = sphere {\n");
   fprintf(outfile,"   <0.0, 0.0, %18.14f>, %18.14f\n",-0.25*l,0.25*l);
   fprintf(outfile,"}\n");

   fprintf(outfile,"\n");
   fprintf(outfile,"\n");

   fprintf(outfile,"#declare c1 = cylinder {\n");
   fprintf(outfile,"   %18.14f*z, 12.0*z, %18.14f\n",-0.25*l,0.25*l);
   fprintf(outfile,"}\n");

   fprintf(outfile,"\n");
   fprintf(outfile,"\n");

   fprintf(outfile,"#declare l3 = union {\n");
   fprintf(outfile,"   object { c1 }\n");
   fprintf(outfile,"   object { s1 }\n");
   fprintf(outfile,"   texture { steeltexture }\n");
   fprintf(outfile,"}\n");

   fprintf(outfile,"\n");
   fprintf(outfile,"\n");

   fprintf(outfile,"#declare l1 = cylinder {\n");
   fprintf(outfile,"   -2.0*z, 2.0*z, 1.0\n");
   fprintf(outfile,"}\n");

   fprintf(outfile,"\n");
   fprintf(outfile,"\n");

   fprintf(outfile,"#declare l2 = intersection {\n");
   fprintf(outfile,"   torus { 2.0, 1.0 }\n");
   fprintf(outfile,"   plane { 1.0*z, 0.0 inverse }\n");
   fprintf(outfile,"}\n");

   fprintf(outfile,"\n");
   fprintf(outfile,"\n");

   fprintf(outfile,"#declare link = union {\n");
   fprintf(outfile,"   object { l1\n");
   fprintf(outfile,"      translate -2.0*x\n");
   fprintf(outfile,"   }\n");
   fprintf(outfile,"   object { l1\n");
   fprintf(outfile,"      translate 2.0*x\n");
   fprintf(outfile,"   }\n");
   fprintf(outfile,"   object { l2\n");
   fprintf(outfile,"      translate 2.0*z\n");
   fprintf(outfile,"   }\n");
   fprintf(outfile,"   object { l2\n");
   fprintf(outfile,"      translate 2.0*z\n");
   fprintf(outfile,"      rotate 180*x\n");
   fprintf(outfile,"   }\n");
   fprintf(outfile,"   rotate 90*x\n");
   fprintf(outfile,"   scale %18.14f\n",0.25*l);
   fprintf(outfile,"   texture { steeltexture }\n");
   fprintf(outfile,"}\n");

   fprintf(outfile,"\n");
   fprintf(outfile,"\n");

   fprintf(outfile,"union {\n");

   fprintf(outfile,"\n");
   fprintf(outfile,"\n");

   fprintf(outfile,"   object { l3\n");
   fprintf(outfile,"      translate %18.14f*x\n",xx1[1]);
   fprintf(outfile,"      translate %18.14f*y\n",yy1[1]);
   fprintf(outfile,"   }\n");

   fprintf(outfile,"\n");
   fprintf(outfile,"\n");

   fprintf(outfile,"   object { l3\n");
   fprintf(outfile,"      translate %18.14f*x\n",xx1[nlinks+1]);
   fprintf(outfile,"      translate %18.14f*y\n",yy1[nlinks+1]);
   fprintf(outfile,"   }\n");

   fprintf(outfile,"\n");
   fprintf(outfile,"\n");

   for (i=1; i<=nlinks; i++) {

      x1=xx1[i];
      y1=yy1[i];

      x2=xx1[i+1];
      y2=yy1[i+1];

      xx=x2-x1;
      yy=y2-y1;

      w=90.0+deg(atan2(yy,xx));

      fprintf(outfile,"   object { link\n");

      if ((i % 2) == 1) {
         fprintf(outfile,"      translate %18.14f*y\n",-0.5*l);
         fprintf(outfile,"      rotate %18.14f*z\n",w);
         fprintf(outfile,"      translate %18.14f*x\n",x1);
         fprintf(outfile,"      translate %18.14f*y\n",y1);
      }
      else {
         fprintf(outfile,"      rotate %18.14f*y\n",90.0);
         fprintf(outfile,"      translate %18.14f*y\n",-l);
         fprintf(outfile,"      rotate %18.14f*z\n",w);
         fprintf(outfile,"      translate %18.14f*x\n",x1);
         fprintf(outfile,"      translate %18.14f*y\n",y1);
      } /* if */

      fprintf(outfile,"   }\n");


   } /* for i */

   fprintf(outfile,"\n");
   fprintf(outfile,"\n");

   fprintf(outfile,"   scale %18.14f\n",scale1);
   fprintf(outfile,"   translate %18.14f*y\n",ytr);
   fprintf(outfile,"   translate %18.14f*z\n",ztr);

   fprintf(outfile,"\n");
   fprintf(outfile,"\n");

   fprintf(outfile,"}\n");


   return(0);


} /* End  */
