/**********************************************************************************/
/*                                                                                */
/*                                                                                */
/* Program                                                                        */
/* =======                                                                        */
/*                                                                                */
/* mkmoeb.c                                                                       */
/*                                                                                */
/*                                                                                */
/* Purpose                                                                        */
/* =======                                                                        */
/*                                                                                */
/* To make POVRAY code for a kind of Moebius band curve (see the files            */
/* moebius*.pov"). The generated code should be pasted into a POVRAY              */ 
/* "blob" object.                                                                 */
/*                                                                                */
/*                                                                                */
/* Version 1.00                                                                   */
/* ============                                                                   */
/*                                                                                */
/* First version: 2006-10-01                                                      */
/* Latest update: 2006-12-08                                                      */
/*                                                                                */
/*                                                                                */
/* Author of the program                                                          */
/* =====================                                                          */
/*                                                                                */
/* Stefan Spaennare, Lund, Sweden                                                 */
/*                                                                                */
/* E-mail: stefan@spaennare.se                                                    */
/*                                                                                */
/* Home page:   http://www.spaennare.se/index.html                                */
/*                                                                                */
/* POVRAY page: http://www.spaennare.se/povray.html                               */
/*                                                                                */
/*                                                                                */
/* Usage                                                                          */
/* =====                                                                          */
/*                                                                                */
/* Parameters ending with "i" are integers and with "r" are real numbers          */
/* (i.e. double precision).                                                       */
/*                                                                                */
/* These parameters work well to define a typical Moebius band curve:             */
/* ------------------------------------------------------------------             */
/*                                                                                */
/* >mkmoeb 1.0 0.5 144 0.2 4.0 1.0 >mo1.pov                                       */
/*                                                                                */
/* Explanation of the parameters:                                                 */
/* ------------------------------                                                 */
/*                                                                                */
/* >Usage: mkmoeb wmaxr wmax2r nwstepsi flatfacr rr r2r                           */
/*                                                                                */
/* wmax: Number of complete revolutions (2*pi) around r the band is formed.       */
/*                                                                                */
/* wmax2: Number of complete revolutions (2*pi) around r2 the band is twisted.    */
/*                                                                                */
/* nwsteps: Number of steps per complete revolution.                              */
/*                                                                                */
/* flatfac: Flatness factor of the band.                                          */
/*                                                                                */
/* r: Major radius of the band.                                                   */
/*                                                                                */
/* r2: Minor radius of the band.                                                  */
/*                                                                                */
/*                                                                                */
/**********************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/


#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

double timetic=(double)(CLOCKS_PER_SEC);

double pi=3.14159265358979323;


double deg(x)
double x;
{
  return(180.0*x/pi);
} /* deg */


int main(argc,argv)
int argc;
char *argv[];
{

   int nw;

   double w,dw,wmax,wmax1,x,y,z,r,r2,h;
   double w2,dw2,wmax2,ff;


   if (argc != 7) {
      printf("Usage: %s wmaxr wmax2r nwstepsi flatfacr rr r2r\n",argv[0]);
      exit(0);
   } /* if */

   wmax1=atof(argv[1]);
   wmax2=atof(argv[2]);
   nw=atoi(argv[3]);
   ff=atof(argv[4]);
   r=atof(argv[5]);
   r2=atof(argv[6]);

   wmax=2.0*pi*wmax1;
   dw=2.0*pi/(double)(nw);
   dw2=2.0*pi*wmax2/(double)(nw);

   h=2.0*r*tan(0.5*dw);

   w=0.0;
   w2=0.0;

   do {

      x=r*cos(w);
      y=0.0;
      z=r*sin(w);

      printf("cylinder { %16.12f*z, %16.12f*z, %16.12f 1\n",-0.5*h,0.5*h,r2);
      printf("   scale <1.0,%16.12f,1.0>\n",ff);
      printf("   rotate %16.12f*z\n",deg(w2));
      printf("   rotate %16.12f*y\n",-deg(w));
      printf("   translate <%16.12f, %16.12f, %16.12f>\n",x,y,z);
      printf("}\n");

      w=w+dw;
      w2=w2+dw2;

   } while (w <= wmax);


   return(0);

} /* End  */
