/**********************************************************************************/
/*                                                                                */
/*                                                                                */
/* Program                                                                        */
/* =======                                                                        */
/*                                                                                */
/* mkmoff.c                                                                       */
/*                                                                                */
/*                                                                                */
/* Purpose                                                                        */
/* =======                                                                        */
/*                                                                                */
/* To make POVRAY code for the Moffat function (see the moff.pov file).           */
/* The code must be present inside a "sor" POVRAY object.                         */
/*                                                                                */
/*                                                                                */
/* Version 1.01                                                                   */
/* ============                                                                   */
/*                                                                                */
/* First version: October 1995                                                    */
/* Latest update: 2006-03-08                                                      */
/*                                                                                */
/*                                                                                */
/* Author of the program                                                          */
/* =====================                                                          */
/*                                                                                */
/* Stefan Spaennare, Lund, Sweden                                                 */
/*                                                                                */
/* E-mail: stefan@spaennare.se                                                    */
/*                                                                                */
/* Home page:   http://www.spaennare.se/index.html                                */
/*                                                                                */
/* POVRAY page: http://www.spaennare.se/povray.html                               */
/*                                                                                */
/*                                                                                */
/* Usage                                                                          */
/* =====                                                                          */
/*                                                                                */
/* Usage: moff2pov widthr heightr fwhmr betar npointsi                            */
/*                                                                                */
/* Parameters ending with "i" are integers and with "r" are real numbers          */
/* (i.e. double precision).                                                       */
/*                                                                                */
/* These parameters work well to define a typical Moffat function:                */
/* ---------------------------------------------------------------                */
/*                                                                                */
/* mkmoff 0.7 1.0 0.4 2.5 100 >temp                                               */
/*                                                                                */
/* The output code must be present inside a "sor" POVRAY object                   */
/* (see the moff.pov file).                                                       */
/*                                                                                */
/* Explanation to the parameters:                                                 */
/* ------------------------------                                                 */
/*                                                                                */
/* width: (0.7); Width (scale factor) of the Moffat function.                     */
/*                                                                                */
/* height: (1.0); Height (scale factor) of the Moffat function.                   */
/*                                                                                */
/* fwhm: (0.4); FWHM (Full Width at Half Maximum) of the Moffat function.         */
/*                                                                                */
/* beta: (2.5); Moffat beta (see description below).                              */
/*                                                                                */
/* npoints: (100); Number of output points to the "sor" POVRAY object.            */
/*                                                                                */
/*                                                                                */
/* The Moffat function                                                            */
/* ===================                                                            */
/*                                                                                */
/* A Moffat function is used in astronomy to fit a function to the image          */
/* of a star in a CCD image taken with a ground based telescope. In many          */
/* cases a Moffat function (Moffat 1969) describes quite well the PSF             */
/* (Point Spread Function) when the light from a star (point source) goes         */
/* through the atmosphere of the Earth. The elliptically symmetric Moffat         */
/* function is given by: m(x,y)=(ax^2+by^2+c)^(-beta). Here a, b and c            */
/* are constants. For circular symmetry a = b. A typically beta value             */
/* is around 2.5.                                                                 */
/*                                                                                */
/*                                                                                */
/**********************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

double timetic=(double)(CLOCKS_PER_SEC);

double pi=3.14159265358979323;


int main(argc,argv)
int argc;
char *argv[];
{

   int i,points,flag;

   double fwhm,gamma,beta,a,w,h,r,z;


   if (argc !=6) {
      printf("Usage: %s widthr heightr fwhmr betar npointsi\n",argv[0]);
      exit(0);
   } /* if */

   w=atof(argv[1]);
   h=atof(argv[2]);
   fwhm=atof(argv[3]);
   beta=atof(argv[4]);
   points=atoi(argv[5]);

   gamma=4.0*(exp(log(2.0)/beta)-1.0);
   a=gamma/(fwhm*fwhm);   

   flag=1;
 
   printf("%8d,\n",points+2);

   for (i=(points-1); i>=1; i--) {

         r=(double)(i)/(double)(points);

         z=exp(-beta*log(1.0+a*r*r));

	 r=w*r;
	 z=h*z;

	 printf("< %16.12lf, %16.12lf>,\n",r,z);

	 if (flag == 1) {
	    printf("< %16.12lf, %16.12lf>,\n",r,z);
         } /* if */

	 flag=0;

   } /* for i */

   
   r=0.0*w;
   z=1.0*h;

    printf("< %16.12lf, %16.12lf>,\n",r,z);
    printf("< %16.12lf, %16.12lf>\n",r,z);

} /* End  */
